# `go generate`命令

`go generate`命令的主要功能是根据代码中的注释，使用注释中声明使用的工具生成所需要的代码文件。首先能够被`go generate`命令识别的注释是有固定格式的，其格式如下。

```go
//go:generate 要执行的命令 命令参数
```

从这个注释的格式可以看出来，执行`go generate`命令实际上就是执行在代码注释中提前编写的命令。这样一来，就比较容易理解`go generate`命令了。我们可以把需要`go generate`执行的命令全部安装起来，然后根据需要编写在所需要的位置，只需要在最终编译项目的时候，提前使用`go generate`命令生成所需要的文件即可。

## 一个生成代码文件的示例

在网上有很多艳尸如何使用`go generate`命令的示例，基本上都大同小异，大多都是选择使用`golang.org/x/tools`工具包中的`cmd/stringer`工具来生成枚举类型的定义。其实还有许多其他的工具也是可以被`go generate`命令使用的，例如生成 Protobuf 数据包结构等。

这里也不免俗的继续用`stringer`工具来演示生成枚举类型定义的功能。在使用`go generate`命令之前，首先需要完成`stringer`工具的安装，可以执行命令`go install golang.org/x/tools/cmd/stringer@latest`来完成。

安装完`stringer`工具以后，我们需要首先来定义一套枚举类型。通常情况下我们定义的枚举类型是需要能够输出其代表内容的。例如如果不使用`stringer`工具生成的时候，手工编写的代码是这样的。

```go
type ErrCode int16

const (
    Err_Authorize_Failed ErrCodes = iota // 用户认证失败
    Err_User_Not_Exists // 用户不存在
    Err_Operate_Denied // 操作被拒绝
    Err_Service_Downgraded // 服务被降级处理
)

func (e ErrCode) String() string {
    switch e {
    case Err_Authorize_Failed:
        return "用户认证失败"
    case Err_User_Not_Exists:
        return "用户不存在"
    case Err_Operate_Denied:
        return "操作被拒绝"
    case Err_Service_Downgraded:
        return "服务被降级处理"
    }
    return "出现未知错误"
}
```

这个文件的内容在每次错误内容发生变化的时候，都需要重新编辑一遍。所以最理想的情况就是我们只需要定义`ErrCode`类型和`const`定义部分，剩下的事情让辅助工具来做。在这里辅助工具就是`stringer`。

在继续之前，首先介绍`stringer`工具中常用的几个选项，这是会被用在`//go:generate`中的。

- `-type`，设定需要生成的内容依据的类型名称，如果需要生成多个类型的内容，则这些类型的名称需要用逗号分隔开。
- `-linecomment`，使用每一行的注释内容作为枚举值输出的内容，不指定这个选项的时候将会直接使用枚举常量本身的内容。
- `-trimprefix 前缀`，移除枚举常量中的前缀内容。

在我们这个示例中，我们只需要用到`-type`和`-linecomment`两个参数。以下是供`stringer`读取的源文件。

```go
package exceptions

//gp:generate stringer -type=ErrCode -linecomment
type ErrCode int16

const (
    Err_Authorize_Failed ErrCodes = iota // 用户认证失败
    Err_User_Not_Exists // 用户不存在
    Err_Operate_Denied // 操作被拒绝
    Err_Service_Downgraded // 服务被降级处理
)
```

这样这个文件就准备好了，现在我们只需要在这个文件所在的目录中执行`go generate`命令，`stringer`工具就会自动生成一个新的文件。这里将这个文件的内容贴出来。

```go
// Code generated by "stringer -type=ErrCode -linecomment"; DO NOT EDIT.

package exceptions

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[Err_Authorize_Failed-0]
	_ = x[Err_User_Not_Exists-1]
	_ = x[Err_Operate_Denied-2]
	_ = x[Err_Service_Downgraded-3]
}

const _ErrCode_name = "用户认证失败用户不存在操作被拒绝服务被降级处理"

var _ErrCode_index = [...]uint8{0, 18, 33, 48, 69}

func (i ErrCode) String() string {
	if i < 0 || i >= ErrCode(len(_ErrCode_index)-1) {
		return "ErrCode(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _ErrCode_name[_ErrCode_index[i]:_ErrCode_index[i+1]]
}
```

可以看到`go generate`调用`stringer`工具根据设定的配置生成了一个经过了优化的代码。
