# 不一样的编程思路

目前世界上比较流行的常用的编程语言，其虽然使用方式不尽相同，但是在了解多了以后，依旧是可以发现其中的共通点的。但是跟 Java、C++这些语言相比，Go 有着其自己独特的设计，这也让 Go 显得十分另类起来。

Go 语言在整体上是十分简单易用的，但是也常常有很多人在诟病 Go 的设计，觉得 Go 应该更多的吸取其他语言中“成熟”的设计。其实 Go 语言在其设计和发展中并非没有借鉴和参考其他语言中的设计和实现，而是 Go 语言的设计团队有其自己设计哲学。

## 少即是多

这一句哲语几乎是 Go 语言设计哲学的核心。我们在实际使用 Go 语言的时候，经常会发现对于一种编程需求的实现，我们没有多少选择，或者说没有其他语言中那种条条大路通罗马的感觉。而少之又少的语言功能，又在很多时候让我们觉得手里的武器捉襟见肘，但又不是完不成任务。

其实一门编程语言不可能包含所有编程者所期望的特性，能够包含这所有特性的语言一定是一个非常复杂的语言，目前能够做到这一点的语言是 C++，我可以保证任何人看到 C++都会觉得这个语言极其的深奥和复杂，这就是包罗万象的结果。

Go 语言所采用的方法更加简单：对于一件事情，只提供一种方法，而且会将这种方法做到极致。这就是 Go 的少即是多的原则。这个原则会大大降低开发者的负担，让开发者能够更加专注在所有实现的业务上，而不是需要分神去考虑如何选择语言所的实现。

所以在使用 Go 语言进行编程的时候，也完全可以遵循这个原则，不必过分的去考虑索要实现的业务功能需要选择怎样的语言特性或者算法特性，直接选择最简单的方法来做就好了。

## 二八定律

“百分之八十的代码仅仅使用到了百分之二十的语言特性”，这也是 Go 语言设计者在其工作中总结出来的一个规律。所以在 Go 语言中，语言特性的增加十分缓慢和谨慎，因为语言特性的增加并不能有效显著的提升开发者的效率，但是却会显著的增加语言的复杂性。

其实这个定律不仅在编程语言的设计中是正确的，在一般的业务项目中也是正确的。大部分的业务项目都存在着过度设计的问题，甚至有一些项目还会为了“炫技”，专门为了设计而设计。这其实都是没有效率的表现。试问，我们在使用 Java 语言设计项目时常用的面向接口的设计方法，在实际的中小型项目中，会有多少情况会真正的被派上用场。因为在绝大多数情况下，一个接口就只有一个实现，这样面向接口的设计方式就有一些过度设计了。也许有人会说，面向接口的设计是为了对未来负责，为未来提供便利，但是大部分项目可能在其整个生命周期里都不会再有什么大的改动了，而且几遍是有改动，可能也会与项目的原始设计差距巨大，提前设计好的接口不一定就一定可以派上用场。这样的话，为什么不在需要的时候再利用局部重构升级所需部分的设计模式呢？

## 万物皆组合

与 Java 中的万物皆对象的世界观不同，Go 选择的设计思路是组合。

继承对于描述世界的思路是集中在一个世界事物中的共通点上，逐步扩大对于这种共通点的描述。但是继承带来的缺点就是，事物是不可能被全面描述的。

但是组合就不一样了，组合其实更加符合世界的构成规律。例如在微观世界中，原子是由质子、中子和电子组成的，分子又是由原子组成的。而在宏观世界中，一个复杂的大型物体也是由不同的零件组成的，即便是形如一体的人，也是由许多执行不同功能的器官组成的。这些器官又是由不同的组织组成的。所以相比之下，组合比继承更加具有表现力。

当然组合也不是没有缺点，组合的缺点就是对于代码的组织和维护，要比面向对象的设计方法差很多，在实际项目中需要定义更加严谨的规范来辅助。

## 静态类型

动态类型语言现在在市面上的应用大行其道，其简便易用的特性让很多人都对静态类型变得不适应了。但是实际上，静态类型的语言在编译期可以执行更加精确地推断和检查，其所带来的好处就是可以极大的避免错误的发生。

例如像 Javascript 编写的程序，对于其中很多稀奇古怪的问题我们都会将其归咎为玄学问题，其实归根结底就是 Javascript 所提供的过大的自由度带来的无法被推断的问题，或者说叫不易被推断的问题。

相比动态类型语言的便捷，静态类型语言虽然牺牲了代码编写的简便，但是提升了运行效率，代码也变得更加易读易懂。

## 没有异常处理

没有异常处理并不是没有异常，Go 语言中还是有异常的。不像是 Java、C++语言中使用的`try ... catch`语法结构，Go 中提供的异常处理是`panic`和`recover`的结构。但是这两种结构几乎是完全的不同。

在使用 Java 语言的`try ... catch`结构的时候，并不是强制的。我们可以选择放任异常的抛出，让程序异常的停下来。或者我们只对我们关心的异常进行了处理，而我们没想到的异常让我们的程序停了下来。

但是 Go 语言中对于异常的处理则是带有一定强制性的，虽然也可以忽略，但是在实际编程中，只要是碰到了返回的异常，还是需要再三的仔细考虑的。

## 天生并发

Go 语言其实就是为并发而生的。对于并发，Go 采用了协程的设计，而不是大多数语言中常见的线程。协程是一种比线程更加轻巧的结构，其创建的时候只使用 4KB 系统内存，这比需要使用 4MB 系统内存的线程已经轻量和非常多了。但是几乎与线程相同的控制模型，有让协程与线程一样易于使用，甚至可能还会更加简单。

所以，在 Java 或者 C++中，我们开了几百个线程就可能已经系统有些吃不消的时候，Go 可以轻轻松松的创建成千上万的协程。想一想这个处理效率吧。
