# 环境安装与配置

Go 语言环境有两种安装方法：安装预编译文件和从源码安装。但是从源码安装并不是构建 Go 语言环境首推的方法，所以这里还是主要介绍安装预编译文件的方法。

## Windows 系统

在 Windows 系统中安装 Go 语言开发环境是非常简单的，在 Go 语言官网提供了专门应用于 Windows 平台的二进制安装包。下载以后只需要双击运行即可。

默认情况下，Go 的二进制文件将会被安装到`C:\Program Files(x86)`目录下面。当然你也可以换一个目录放置，但是这样你就需要在接下来的环境变量配置中使用你所选择的新的目录。

完成安装以后，如果能够成功执行命令`go version`，那么就基本完成了 Go 语言环境的安装。

## Linux 系统

Linux 系统中的 Go 开发环境安装就逼近简单粗暴了，在从 Go 官网下载适用于当前系统的压缩包以后，需要在压缩包所在目录中执行以下命令来完成安装。

```bash
rm -rf /usr/local/go && tar -C /usr/local -xzf go1.20.4.linux-amd64.tar.gz
```

这个命令将删除系统中之前安装过的 Go 语言环境，然后再将新版本的环境安装进去。这个命令的用法不仅可以用于 Go 语言环境的全新安装，还可以用于 Go 语言环境的升级。

安装结束以后需要确认一下系统中的`PATH`环境变量的设置，需要将 Go 环境目录引入进来。如果需要设置可以将以下内容写入到系统配置中（通常是`$HOME/.profile`或者`/etc/profile`）。

```bash
export PATH=$PATH:/usr/local/go/bin
```

与 Windows 系统中的安装确认一样，要确认 Linux 系统中是否已经完成了 Go 语言环境的安装，也同样可以执行`go version`命令，如果能把正常显示所安装的 Go 语言的版本号，那么就说明 Go 语言环境已经基本完成安装了。

## macOS 系统

macOS 系统中的 Go 语言环境安装就要简单多了。除了可以像 Linux 系统中那样安装以外，还可以通过 Homebrew 软件管理器来安装。只需要在终端中执行以下命令即可。

```bash
brew install go
```

Homebrew 会自动下载所需要的 Go 的语言环境压缩包，之后会将其解压到其所管理的位置，最后还会自动完成`PATH`环境变量的设置。在一般情况下，当 Homebrew 命令执行结束，`go version`命令就可以正常执行了。

## 环境变量配置

Go 语言不管是命令还是程序执行都需要一些环境变量的支持，Go 语言所使用的环境变量可以通过命令`go env`来查看。虽然这个命令会列出很多环境变量，但是其中对我们比较重要的只有少数的几个。

```admonish tip
`go env`命令只能列出系统中目前已经设置的环境变量，不能用来设置新的环境变量。如果需要设置环境变量的值，还是需要到`$HOME/.profile`或者`/etc/profile`文件中。
```

常见的环境变量并且需要设置的环境变量主要有以下几个。

### `GO111MODULE`

用于指示 Go 是否开启对`go.mod`文件检测以使用 Module 依赖管理的依据。这个环境变量默认为空，可以设置为`on`来使用 Go 1.13 中正式启用的 Module 依赖分析。设置为`off`可以继续使用 Go 1.13 之前的项目管理风格。

### `GOPROXY`

用于设置 Go 的全局代理。默认情况下，Go 会从 Github 下载依赖包，但是对于国内环境来说，Github 并不能保证稳定访问，所以就需要设置这一环境变量来启用 Go 的全局代理。Go 的默认全局代理设置为`https://proxy.golang.org`对于国内来说可以选择以下几个代理：

- `https://goproxy.cn`，七牛云的代理
- `https://mirrors.aliyun.com/goproxy/`，阿里云的代理
- `https://goproxy.io/`，官方全球代理

例如需要设置使用七牛云的代理，只需要这样设置环境变量。

```bash
GOPROXY=https://goproxy.cn,direct
```

通过`GOPROXY`可以设置多个代理，所需要使用的代理只需要使用逗号分开即可。但代理设置的最后一项应该确保为`direct`，这样才能够使 Go 在下载依赖包的时候，能够使用直连的方式访问。如果将`GOPROXY`的值设置为`off`，那么将关闭 Go 对于使用代理服务器的支持。

### `GOSUMDB`

常常与`GOPROXY`一起配置的环境变量就是`GOSUMDB`了。`GOSUMDB`环境变量的出现主要是为了解决 Go 在获取依赖包的时候存在的安全问题。在 Go Module 功能被引入之后，包依赖的安全问题就进入了 Go 语言开发团队的视野。Go 在解决依赖包安全的问答题上采用的方法是对每一个依赖包进行校验和（checksum）计算，并将计算得到的校验和记录在一个数据库里，当 Go 下载依赖包以后，也会对下载的依赖包进行校验和计算，并将计算结果与之前记录在数据库中的校验和进行对比。如果对比的结果是一致的，那么就说明下载的依赖包是安全可信的。

当然 Go 不是在每次编译的时候都会去重新计算校验和，而使会将计算过的校验和保存在项目根目录中一个名为`go.sum`的文件中。这样当项目编译的时候，其所使用的依赖包的校验和与`go.sum`中记录的校验和不一致，那么 Go 就会报出一个提示，来提醒你所下载的内容已经发生了变化。

环境变量`GOSUMDB`中配置的内容是校验和数据库服务器所在和访问公钥。例如：

```bash
GOSUMDB="gosum.io+ce6e7565+AY5qEHUk/qmHc5btzW45JVoENfazw8LielDsaI+lEbq6"
```

在默认情况下，当`GOPROXY`被设置为`GOPROXY="https://proxy.golang.org"`的时候，`GOSUMDB`就会默认的使用`sum.golang.org`。但是对于身处中国的开发者来说，这个地址依然也是不能稳定访问的，所以，我们要么需要将`GOSUMDB`环境变量设置为`off`，要么就要设置一个可以稳定访问的代理。

例如常用的校验和数据库地址主要有两个：

- `sum.golang.google.cn`，这是 Golang 专门为中国推出的校验和数据库镜像。
- `gosum.io`，这是 goproxy.io 推出的校验和数据库镜像。

```admonish tip
不推荐在使用`go get`命令下载依赖包的时候使用`-insecure`参数来略过依赖包的安全检测，这是一种非常不安全的行为。
```

### `GOPATH`

这是一个几乎在任何 Go 语言资料中都可以见到的环境变量，它用来指定 Go 项目工作区的位置，这个工作区中会保存源代码、测试文件、库文件、可执行文件等。

但是在 Go 1.11 版本引入 Module 特性以后，`GOPATH`环境变量的重要性就降低了。首先在使用 Module 模式开发项目的时候，项目已经不必必须放置在`$GOPATH/src`目录下了，但是`go mod`命令下载的依赖模块依旧会保存在`$GOPATH/pkg/mod`下。

### `GOROOT`

`GOROOT`变量在很多 Go 的介绍中都有出现，这个环境变量指定了 Go 的安装目录，如果修改了这个变量的内容，那么将会有很大的可能会使得 Go 命令无法执行。

```admonish tip
如果在修改`GOROOT`以后发生了Go命令找不到而无法使用的情况，那么可以将自己编辑的配置文件中的`GOROOT`环境变量内容删除，即可回复`GOROOT`环境变量的默认值。
```

### `GOMODCACHE`

`GOMODCACHE`这个环境变量看它的名字就能大概知道这个环境变量是用来定义缓存内容位置的。这个环境变量所定义的内容是 Go 在下载外部依赖文件的时候，存放外部依赖文件的缓存目录。在一般情况下这个环境变量并不用特别的设置，Go 会默认把外部依赖文件下载到`$GOPATH/pkg/mod`下。所以如果更改了`GOPATH`环境变量的值，那么`GOMODCACHE`的内容也会跟随发生变化。

### `GOPRIVATE`

在默认情况下，Go 会从公共镜像上下载项目所有的依赖包，并且会对所下载的内容进行安全校验。但是在实际项目开发的时候，并不是所有的代码都是公开的，也就是说有一部分依赖包是保存在私有仓库中的，在这种情况下，通过公共的镜像就不可能访问到这些私有内容了。所以就需要设定`GOPRIVATE`环境变量来告诉 Go，哪些依赖包是需要被作为私有库处理。

`GOPRIVATE`环境变量可以接受多个值，各个值之间只需要使用逗号分隔开即可。`GOPRIVATE`中可以使用的值的形式只需要能够匹配`path.Match()`即可，换句话说，就是不必在`GOPRIVATE`中逐个列举私有的每一个库，只需要列出其共通的部分即可。例如私有项目通常会放在私有的 Git 库里，所以就可以将私有 Git 库地址放入`GORIVATE`。

```bash
GOPRIVATE=git.archgrid.xyz/go_module
```

在这个示例中，所有存储在`git.archgrid.xyz/go_module`下的模块在获取的时候就不会再走`GOPROXY`设置的代理了。

### 总结

在完成 Go 语言环境安装以后，我们可能需要设置的环境变量主要有以下几个。

```bash
GOPATH=/path/to/your/cache/
GO111MODULE=on
GOPROXY=https://goproxy.io,direct
GOSUMDB=gosum.io+ce6e7565+AY5qEHUk/qmHc5btzW45JVoENfazw8LielDsaI+lEbq6
```

完成这几个环境变量的配给，接下来就可以开始 Go 语言程序的编写了。
